package com.smcic.api.admin.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.smcic.api.admin.entity.AdminRole;
import com.smcic.api.admin.entity.MenuModule;
import com.smcic.api.admin.entity.RoleAclModule;
import com.smcic.api.admin.mapper.RoleAclModuleMapper;
import com.smcic.api.admin.service.IRoleAclModuleService;
import com.smcic.core.advice.APIException;
import com.smcic.core.auth.AESUtil;
import com.smcic.core.auth.HttpContextUtil;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 角色模块权限表 服务实现类
 * </p>
 *
 * @author syj
 * @since 2023-11-16
 */
@Service
public class RoleAclModuleServiceImpl extends ServiceImpl<RoleAclModuleMapper, RoleAclModule> implements IRoleAclModuleService {

    @Resource
    private MenuModuleServiceImpl menuModuleService;

    @Resource
    private AdminUserRoleServiceImpl adminUserRoleService;

    public List<MenuModule> getModules(Long roleId) {
        List<Long> moduleIds = lambdaQuery().eq(RoleAclModule::getRoleId, roleId).list().stream().map(RoleAclModule::getModuleId).collect(Collectors.toList());

        if (moduleIds.contains( -1L)) {
            return menuModuleService.list();
        }

        List<MenuModule> menuModules = menuModuleService.lambdaQuery().in(MenuModule::getModuleId, moduleIds).list();

        if(!menuModules.isEmpty()) {
            List<Long> pids =  menuModules.stream().map(MenuModule::getPid).distinct().filter(id -> !moduleIds.contains(id)).collect(Collectors.toList());
            if(!pids.isEmpty()) {
                List<MenuModule> pList = menuModuleService.lambdaQuery().in(
                        MenuModule::getModuleId, pids
                ).list();
                menuModules.addAll(pList);
            }
        }

        Map<Long, MenuModule> menuModuleMap = menuModules.stream().collect(Collectors.toMap(MenuModule::getModuleId, menuModule -> menuModule));

        List<MenuModule> rs = new ArrayList<>();
        for (MenuModule menuModule: menuModules){
            if(menuModule.getPid() == 0){
                rs.add(menuModule);
            }else{
                MenuModule parent = menuModuleMap.get(menuModule.getPid());
                parent.getChildren().add(menuModule);
            }
        }
        return rs;

    }

    public List<String> getCurrAllowPath(){
        if(!HttpContextUtil.getToken().isPresent()){
            throw new APIException("请先登录");
        }
        Long uid = Long.valueOf(AESUtil.decryptStr(HttpContextUtil.getToken().get()));
        List<AdminRole> roles = adminUserRoleService.getRoles(uid);
        List<MenuModule> modules = new ArrayList<>();
        for (AdminRole adminRole: roles){
            modules.addAll(getModules(adminRole.getRoleId()));
        }
        return modules.stream().map(MenuModule::getRouter).distinct().collect(Collectors.toList());
    }
}
