package com.smcic.api.admin.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.smcic.api.admin.dto.DeviceDTO;
import com.smcic.api.admin.entity.UserDevice;
import com.smcic.api.admin.mapper.UserDeviceMapper;
import com.smcic.api.admin.service.IUserDeviceService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.smcic.core.advice.APIException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author syj
 * @since 2023-11-16
 */
@Service
public class UserDeviceServiceImpl extends ServiceImpl<UserDeviceMapper, UserDevice> implements IUserDeviceService {

    @Transactional
    public void add(DeviceDTO deviceDTO){
        List<String> list = lambdaQuery().eq(UserDevice::getUserId, deviceDTO.getUserId()).list()
                .stream().map(UserDevice::getDeviceCode).collect(Collectors.toList());

        List<String> codes = deviceDTO.getDeviceCodes().stream().filter(deviceCode -> !list.contains(deviceCode)).collect(Collectors.toList());
        if(codes.isEmpty()) {
            throw new APIException("请填入新的设备");
        }
        int MAX_DEVICE = 3;
        if(list.size() + codes.size() > MAX_DEVICE) {
            throw new APIException("设备数量超过上限");
        }
        boolean b = saveBatch(codes.stream().map(deviceCode -> {
            UserDevice userDevice = new UserDevice();
            userDevice.setUserId(deviceDTO.getUserId());
            userDevice.setDeviceCode(deviceCode);
            return userDevice;
        }).collect(Collectors.toList()));
        if(!b) {
            throw new APIException("添加失败");
        }
    }

    public void delete(DeviceDTO deviceDTO){
        LambdaQueryWrapper<UserDevice> lambdaQueryWrapper = Wrappers.lambdaQuery();
        remove(lambdaQueryWrapper.eq(UserDevice::getUserId, deviceDTO.getUserId()).in(UserDevice::getDeviceCode, deviceDTO.getDeviceCodes()));
    }

}
